@@ -19,6 +19,7 @@
 #define WEBPAGE_H
 
 #include <QWebEnginePage>
+#include <QWebEngineScript>
 #include <QWebEngineFullScreenRequest>
 #include <QVector>
 
@@ -35,18 +36,30 @@ class DelayedFileWatcher;
 class QUPZILLA_EXPORT WebPage : public QWebEnginePage
 {
     Q_OBJECT
+
 public:
+    enum JsWorld {
+        SafeJsWorld = QWebEngineScript::ApplicationWorld
+    };
+
     explicit WebPage(QObject* parent = 0);
     ~WebPage();
 
     WebView *view() const;
 
-    QVariant execJavaScript(const QString &scriptSource, int timeout = 500);
+    QVariant execJavaScript(const QString &scriptSource, quint32 worldId = QWebEngineScript::MainWorld, int timeout = 500);
 
-    QPoint mapToViewport(const QPoint &pos) const;
+    // TODO: Remove when depending on Qt 5.7
+    void runJavaScript(const QString &scriptSource);
+    void runJavaScript(const QString &scriptSource, const QWebEngineCallback<const QVariant &> &resultCallback);
+    void runJavaScript(const QString &scriptSource, quint32 worldId);
+    void runJavaScript(const QString &scriptSource, quint32 worldId, const QWebEngineCallback<const QVariant &> &resultCallback);
+
+    QPointF mapToViewport(const QPointF &pos) const;
     WebHitTestResult hitTestContent(const QPoint &pos) const;
 
     void scroll(int x, int y);
+    void setScrollPosition(const QPointF &pos);
 
     bool javaScriptPrompt(const QUrl &securityOrigin, const QString &msg, const QString &defaultValue, QString* result) Q_DECL_OVERRIDE;
     bool javaScriptConfirm(const QUrl &securityOrigin, const QString &msg) Q_DECL_OVERRIDE;
